"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeOrmFormatErrorService = exports.PostgresError = exports.formErrorString = void 0;
const tslib_1 = require("tslib");
const json_api_nestjs_1 = require("@klerick/json-api-nestjs");
const common_1 = require("@nestjs/common");
const typeorm_1 = require("typeorm");
const DriverUtils_1 = require("typeorm/driver/DriverUtils");
const constants_1 = require("../constants");
const formErrorString = (entityMetadata, errorText) => {
    for (const column of entityMetadata.columns) {
        const result = new RegExp(column.databaseName).test(errorText);
        if (!result)
            continue;
        errorText = errorText.replace(column.databaseName, column.propertyName);
    }
    return errorText.replace(entityMetadata.tableName, entityMetadata.name);
};
exports.formErrorString = formErrorString;
const fieldNotNullOrDefault = (entityMetadata, errorText, detail) => {
    const error = {
        code: 'invalid_arguments',
        message: (0, exports.formErrorString)(entityMetadata, errorText),
        path: ['data', 'attributes'],
    };
    return new common_1.BadRequestException([error]);
};
const duplicateItems = (entityMetadata, errorText, detail) => {
    errorText = 'Duplicate value';
    if (detail) {
        const matches = detail.match(/(?<=\().+?(?=\))/gm);
        if (matches) {
            errorText = `Duplicate value in the "${matches[0]}"`;
        }
    }
    const error = {
        code: 'invalid_arguments',
        message: detail ? (0, exports.formErrorString)(entityMetadata, errorText) : errorText,
        path: ['data', 'attributes'],
    };
    return new common_1.ConflictException([error]);
};
const invalidInputSyntax = (entityMetadata, errorText, detail) => {
    const error = {
        code: 'invalid_arguments',
        message: errorText,
        path: [],
    };
    return new common_1.BadRequestException([error]);
};
const entityHasRelation = (entityMetadata, errorText, detail) => {
    const error = {
        code: 'invalid_arguments',
        message: detail || errorText,
        path: ['data', 'attributes'],
    };
    return new common_1.NotAcceptableException([error]);
};
exports.PostgresError = {
    [23502]: fieldNotNullOrDefault,
    [23505]: duplicateItems,
    ['22P02']: invalidInputSyntax,
    [22007]: invalidInputSyntax,
    [22003]: invalidInputSyntax,
    [23503]: entityHasRelation,
};
class TypeOrmFormatErrorService extends json_api_nestjs_1.ErrorFormatService {
    entityManager;
    mapProperty;
    _dbDriver = undefined;
    get dbDriver() {
        if (!this._dbDriver) {
            const firstEntity = [...this.mapProperty.keys()].at(0);
            if (!firstEntity) {
                throw new Error('No entity found in map');
            }
            this._dbDriver =
                this.entityManager.getRepository(firstEntity).manager.connection.driver;
        }
        return this._dbDriver;
    }
    formatError(error) {
        if (!(error instanceof typeorm_1.QueryFailedError)) {
            return super.formatError(error);
        }
        try {
            return this.prepareDataBaseError(error);
        }
        catch (error) {
            return super.formatError(error);
        }
    }
    prepareDataBaseError(error) {
        if (DriverUtils_1.DriverUtils.isPostgresFamily(this.dbDriver)) {
            const { errorCode, errorMsg, detail, table } = {
                errorCode: error.driverError
                    .code,
                errorMsg: error.driverError.message,
                detail: error.driverError.detail,
                table: error.driverError.table,
            };
            const metadata = this.entityManager.connection.entityMetadatas.find((i) => i.tableName === table);
            if (!metadata) {
                return super.formatError(error);
            }
            if (exports.PostgresError[errorCode]) {
                return exports.PostgresError[errorCode](metadata, errorMsg, detail);
            }
        }
        return super.formatError(error);
    }
}
exports.TypeOrmFormatErrorService = TypeOrmFormatErrorService;
tslib_1.__decorate([
    (0, common_1.Inject)(constants_1.CURRENT_ENTITY_MANAGER_TOKEN),
    tslib_1.__metadata("design:type", typeorm_1.EntityManager)
], TypeOrmFormatErrorService.prototype, "entityManager", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(json_api_nestjs_1.ENTITY_PARAM_MAP),
    tslib_1.__metadata("design:type", Map)
], TypeOrmFormatErrorService.prototype, "mapProperty", void 0);
//# sourceMappingURL=type-orm-format.error.service.js.map